@@ -152,6 +152,8 @@ class CFGBase(Analysis):
         self._node_lookup_index = None
         self._node_lookup_index_warned = False
 
+        self._function_addresses_from_symbols = self._func_addrs_from_symbols()
+
         if model is not None:
             self._model = model
         else:
@@ -268,12 +270,12 @@ class CFGBase(Analysis):
         return self._model.get_successors_and_jumpkind(node, excluding_fakeret=excluding_fakeret)
 
     @deprecated(replacement="self.model.get_all_predecessors()")
-    def get_all_predecessors(self, cfgnode):
-        return self._model.get_all_predecessors(cfgnode)
+    def get_all_predecessors(self, cfgnode, depth_limit=None):
+        return self._model.get_all_predecessors(cfgnode, depth_limit)
 
     @deprecated(replacement="self.model.get_all_successors()")
-    def get_all_successors(self, cfgnode):
-        return self._model.get_all_successors(cfgnode)
+    def get_all_successors(self, cfgnode, depth_limit=None):
+        return self._model.get_all_successors(cfgnode, depth_limit)
 
     @deprecated(replacement="self.model.get_node()")
     def get_node(self, block_id):
@@ -718,6 +720,16 @@ class CFGBase(Analysis):
         except KeyError:
             return None
 
+    def _func_addrs_from_symbols(self):
+        """
+        Get all possible function addresses that are specified by the symbols in the binary
+
+        :return: A set of addresses that are probably functions
+        :rtype:  set
+        """
+
+        return {sym.rebased_addr for sym in self._binary.symbols if sym.is_function}
+
     #
     # Analyze function features
     #
@@ -1256,13 +1268,15 @@ class CFGBase(Analysis):
         # aggressively remove and merge functions
         # For any function, if there is a call to it, it won't be removed
         called_function_addrs = { n.addr for n in function_nodes }
+        # Any function addresses that appear as symbols won't be removed
+        predetermined_function_addrs = called_function_addrs | self._function_addresses_from_symbols
 
         removed_functions_a = self._process_irrational_functions(tmp_functions,
-                                                                 called_function_addrs,
+                                                                 predetermined_function_addrs,
                                                                  blockaddr_to_function
                                                                  )
         removed_functions_b, adjusted_cfgnodes = self._process_irrational_function_starts(tmp_functions,
-                                                                                          called_function_addrs,
+                                                                                          predetermined_function_addrs,
                                                                                           blockaddr_to_function
                                                                                           )
         removed_functions = removed_functions_a | removed_functions_b
@@ -1349,8 +1363,8 @@ class CFGBase(Analysis):
 
     def _process_irrational_functions(self, functions, predetermined_function_addrs, blockaddr_to_function):
         """
-        For unresolveable indirect jumps, angr marks those jump targets as individual functions. For example, usually
-        the following pattern is seen:
+        When force_complete_scan is enabled, for unresolveable indirect jumps, angr will find jump targets and mark
+        them as individual functions. For example, usually the following pattern is seen:
 
         sub_0x400010:
             push ebp
@@ -1381,9 +1395,9 @@ class CFGBase(Analysis):
 
         functions_to_remove = { }
 
-        functions_can_be_removed = set(functions.keys()) - set(predetermined_function_addrs)
+        all_func_addrs = sorted(set(functions.keys()))
 
-        for func_addr, function in functions.items():
+        for func_pos, (func_addr, function) in enumerate(functions.items()):
 
             if func_addr in functions_to_remove:
                 continue
@@ -1440,6 +1454,10 @@ class CFGBase(Analysis):
             tmp_state = self.project.factory.blank_state(mode='fastpath')
             while True:
                 try:
+                    # do not follow hooked addresses (such as SimProcedures)
+                    if self.project.is_hooked(last_addr):
+                        break
+
                     # using successors is slow, but acceptable since we won't be creating millions of blocks here...
                     tmp_state.ip = last_addr
                     b = self.project.factory.successors(tmp_state, jumpkind='Ijk_Boring')
@@ -1465,7 +1483,10 @@ class CFGBase(Analysis):
 
             should_merge = True
             functions_to_merge = set()
-            for f_addr in functions_can_be_removed:
+            i = func_pos + 1
+            while i < len(all_func_addrs):
+                f_addr = all_func_addrs[i]
+                i += 1
                 f = functions[f_addr]
                 if f_addr == func_addr:
                     continue
@@ -1558,6 +1579,9 @@ class CFGBase(Analysis):
                 cfgnode_0 = self.model.get_any_node(addr_0)
                 cfgnode_1 = self.model.get_any_node(addr_1)
 
+                if cfgnode_0 is None or cfgnode_1 is None:
+                    continue
+
                 # Are func_0 adjacent to func_1?
                 if cfgnode_0.addr + cfgnode_0.size != addr_1:
                     continue
@@ -1642,9 +1666,10 @@ class CFGBase(Analysis):
         :return:    True if it is a tail-call optimization. False otherwise.
         :rtype:     bool
         """
-
         def _has_more_than_one_exit(node_):
-            return len(g.out_edges(node_)) > 1
+            # Do not consider FakeRets as counting as multiple exits here.
+            out_edges = list(filter(lambda x: g.get_edge_data(*x)['jumpkind'] != 'Ijk_FakeRet', g.out_edges(node_)))
+            return len(out_edges) > 1
 
         if len(all_edges) == 1 and dst_addr != src_addr:
             the_edge = next(iter(all_edges))
@@ -1988,16 +2013,25 @@ class CFGBase(Analysis):
         :return: True if the instruction does no-op, False otherwise.
         """
 
-        if insn.insn_name() == 'nop':
+        insn_name = insn.insn_name()
+
+        if insn_name == 'nop':
             # nops
             return True
-        if insn.insn_name() == 'lea':
+        elif insn_name == 'lea':
             # lea reg, [reg + 0]
             op0, op1 = insn.operands
             if op0.type == 1 and op1.type == 3:
                 # reg and mem
                 if op0.reg == op1.mem.base and op1.mem.index == 0 and op1.mem.disp == 0:
                     return True
+        elif insn_name == 'mov':
+            # mov reg_a, reg_a
+            op0, op1 = insn.operands
+            if op0.type == 1 and op1.type == 1:
+                # reg and reg
+                if op0.reg == op1.reg:
+                    return True
 
         # add more types of no-op instructions here :-)
 
@@ -2137,6 +2171,10 @@ class CFGBase(Analysis):
         # Create an IndirectJump instance
         if addr not in self.indirect_jumps:
             if self.project.arch.branch_delay_slot:
+                if len(cfg_node.instruction_addrs) < 2:
+                    # sanity check
+                    # decoding failed when decoding the second instruction (or even the first instruction)
+                    return False, [ ], None
                 ins_addr = cfg_node.instruction_addrs[-2]
             else:
                 ins_addr = cfg_node.instruction_addrs[-1]
