@@ -5,7 +5,7 @@ from collections import defaultdict, OrderedDict
 import pyvex
 from archinfo.arch_arm import is_arm_arch
 
-from ....engines.vex import ccall
+from angr.engines.vex.claripy import ccall
 from ....engines.light import SimEngineLightVEXMixin, SimEngineLight, SpOffset, RegisterOffset
 from ....errors import AngrError, SimError
 from ....blade import Blade
@@ -193,47 +193,23 @@ class JumpTableProcessor(
             self.state._registers[expr.offset] = ([src], v)
             return v
 
+    def _handle_function(self, expr):  # pylint:disable=unused-argument,no-self-use
+        return None  # This analysis is not interprocedural
+
     def _handle_Load(self, expr):
         addr = self._expr(expr.addr)
         size = expr.result_size(self.tyenv) // 8
+        return self._do_load(addr, size)
 
-        src = (self.block.addr, self.stmt_idx)
-        self._tsrc = { src }
-        if addr is None:
+    def _handle_LoadG(self, stmt):
+        guard = self._expr(stmt.guard)
+        if guard is True:
+            return self._do_load(stmt.addr, stmt.addr.result_size(self.tyenv) // 8)
+        elif guard is False:
+            return self._do_load(stmt.alt, stmt.alt.result_size(self.tyenv) // 8)
+        else:
             return None
 
-        if isinstance(addr, SpOffset):
-            if addr.offset in self.state._stack:
-                self._tsrc = { self.state._stack[addr.offset][0] }
-                return self.state._stack[addr.offset][1]
-        elif isinstance(addr, int):
-            # Load data from memory if it is mapped
-            try:
-                v = self.project.loader.memory.unpack_word(addr, size=size)
-                return v
-            except KeyError:
-                return None
-        elif isinstance(addr, RegisterOffset):
-            # Load data from a register, but this register hasn't been initialized at this point
-            # We will need to initialize this register during slice execution later
-
-            # Try to get where this register is first accessed
-            try:
-                source = next(iter(src for src in self.state._registers[addr.reg][0] if src != 'const'))
-                assert isinstance(source, tuple)
-                self.state.regs_to_initialize.append(source + (addr.reg, addr.bits))
-            except StopIteration:
-                # we don't need to initialize this register
-                # it might be caused by an incorrect analysis result
-                # e.g.  PN-337140.bin 11e918  r0 comes from r4, r4 comes from r0@11e8c0, and r0@11e8c0 comes from
-                # function call sub_375c04. Since we do not analyze sub_375c04, we treat r0@11e918 as a constant 0.
-                pass
-
-
-            return None
-
-        return None
-
     def _handle_Const(self, expr):
         v = super()._handle_Const(expr)
         self._tsrc.add('const')
@@ -347,6 +323,43 @@ class JumpTableProcessor(
                 #
                 self.state.stmts_to_instrument.append(('reg_write', ) + arg1_src)
 
+    def _do_load(self, addr, size):
+        src = (self.block.addr, self.stmt_idx)
+        self._tsrc = { src }
+        if addr is None:
+            return None
+
+        if isinstance(addr, SpOffset):
+            if addr.offset in self.state._stack:
+                self._tsrc = { self.state._stack[addr.offset][0] }
+                return self.state._stack[addr.offset][1]
+        elif isinstance(addr, int):
+            # Load data from memory if it is mapped
+            try:
+                v = self.project.loader.memory.unpack_word(addr, size=size)
+                return v
+            except KeyError:
+                return None
+        elif isinstance(addr, RegisterOffset):
+            # Load data from a register, but this register hasn't been initialized at this point
+            # We will need to initialize this register during slice execution later
+
+            # Try to get where this register is first accessed
+            try:
+                source = next(iter(src for src in self.state._registers[addr.reg][0] if src != 'const'))
+                assert isinstance(source, tuple)
+                self.state.regs_to_initialize.append(source + (addr.reg, addr.bits))
+            except StopIteration:
+                # we don't need to initialize this register
+                # it might be caused by an incorrect analysis result
+                # e.g.  PN-337140.bin 11e918  r0 comes from r4, r4 comes from r0@11e8c0, and r0@11e8c0 comes from
+                # function call sub_375c04. Since we do not analyze sub_375c04, we treat r0@11e918 as a constant 0.
+                pass
+
+            return None
+
+        return None
+
 
 #
 # State hooks
@@ -633,6 +646,9 @@ class JumpTableResolver(IndirectJumpResolver):
                 break
             block_addr, stmt_idx = stmt_loc = preds[0]
             block = project.factory.block(block_addr, backup_state=self.base_state).vex
+            if stmt_idx == DEFAULT_STATEMENT:
+                # it's the default exit. continue
+                continue
             stmt = block.statements[stmt_idx]
             if isinstance(stmt, (pyvex.IRStmt.WrTmp, pyvex.IRStmt.Put)):
                 if isinstance(stmt.data, (pyvex.IRExpr.Get, pyvex.IRExpr.RdTmp)):
@@ -833,6 +849,8 @@ class JumpTableResolver(IndirectJumpResolver):
         :rtype:     tuple of lists
         """
 
+        # pylint:disable=no-else-continue
+
         engine = JumpTableProcessor(self.project)
 
         sources = [ n for n in b.slice.nodes() if b.slice.in_degree(n) == 0 ]
@@ -845,7 +863,7 @@ class JumpTableResolver(IndirectJumpResolver):
             traced = { src[0] }
             while src is not None:
                 state._tmpvar_source.clear()
-                block_addr, stmt_idx = src
+                block_addr, _ = src
 
                 block = self.project.factory.block(block_addr, backup_state=self.base_state)
                 stmt_whitelist = annotatedcfg.get_whitelisted_statements(block_addr)
@@ -1063,6 +1081,10 @@ class JumpTableResolver(IndirectJumpResolver):
             mask = (2 ** self.project.arch.bits) - 1
             all_targets = [(target + base_addr) & mask for target in all_targets]
 
+        # special case for ARM: if the source block is in THUMB mode, all jump targets should be in THUMB mode, too
+        if is_arm_arch(self.project.arch) and (addr & 1) == 1:
+            all_targets = [ target | 1 for target in all_targets ]
+
         # Finally... all targets are ready
         illegal_target_found = False
         for target in all_targets:
@@ -1095,25 +1117,25 @@ class JumpTableResolver(IndirectJumpResolver):
             if sort == 'mem_write':
                 bp = BP(when=BP_BEFORE, enabled=True, action=StoreHook.hook,
                         condition=lambda _s, a=block_addr, idx=stmt_idx:
-                            _s.scratch.bbl_addr == a and _s.scratch.stmt_idx == idx
+                            _s.scratch.bbl_addr == a and _s.inspect.statement == idx
                         )
                 state.inspect.add_breakpoint('mem_write', bp)
             elif sort == 'mem_read':
                 hook = LoadHook()
                 bp0 = BP(when=BP_BEFORE, enabled=True, action=hook.hook_before,
                          condition=lambda _s, a=block_addr, idx=stmt_idx:
-                            _s.scratch.bbl_addr == a and _s.scratch.stmt_idx == idx
+                            _s.scratch.bbl_addr == a and _s.inspect.statement == idx
                          )
                 state.inspect.add_breakpoint('mem_read', bp0)
                 bp1 = BP(when=BP_AFTER, enabled=True, action=hook.hook_after,
                          condition=lambda _s, a=block_addr, idx=stmt_idx:
-                            _s.scratch.bbl_addr == a and _s.scratch.stmt_idx == idx
+                            _s.scratch.bbl_addr == a and _s.inspect.statement == idx
                          )
                 state.inspect.add_breakpoint('mem_read', bp1)
             elif sort == 'reg_write':
                 bp = BP(when=BP_BEFORE, enabled=True, action=PutHook.hook,
                         condition=lambda _s, a=block_addr, idx=stmt_idx:
-                            _s.scratch.bbl_addr == a and _s.scratch.stmt_idx == idx
+                            _s.scratch.bbl_addr == a and _s.inspect.statement == idx
                         )
                 state.inspect.add_breakpoint('reg_write', bp)
             else:
@@ -1125,7 +1147,7 @@ class JumpTableResolver(IndirectJumpResolver):
                     state.arch.translate_register_name(reg_offset, size=reg_bits),
                     block_addr, stmt_idx)
             bp = BP(when=BP_BEFORE, enabled=True, action=RegisterInitializerHook(reg_offset, reg_bits, reg_val).hook,
-                    condition=lambda _s: _s.scratch.bbl_addr == block_addr and _s.scratch.stmt_idx == stmt_idx
+                    condition=lambda _s: _s.scratch.bbl_addr == block_addr and _s.inspect.statement == stmt_idx
                     )
             state.inspect.add_breakpoint('statement', bp)
             reg_val += 16
