@@ -469,24 +469,30 @@ class SimSymbolicMemory(SimMemory): #pylint:disable=abstract-method
                         "to suppress these messages.")
 
                 if is_mem:
-                    refplace_int = self.state.solver.eval(self.state._ip)
-                    if self.state.project:
-                        refplace_str = self.state.project.loader.describe_addr(refplace_int)
-                    else:
-                        refplace_str = "unknown"
-                    l.warning("Filling memory at %#x with %d unconstrained bytes referenced from %#x (%s)", addr, num_bytes, refplace_int, refplace_str)
-                else:
-                    if addr == self.state.arch.ip_offset:
-                        refplace_int = 0
-                        refplace_str = "symbolic"
-                    else:
+                    refplace_str = "unknown"
+                    if not self.state._ip.symbolic:
                         refplace_int = self.state.solver.eval(self.state._ip)
+                        refplace_int_s = "%#x" % refplace_int
                         if self.state.project:
                             refplace_str = self.state.project.loader.describe_addr(refplace_int)
+                    else:
+                        refplace_int_s = repr(self.state._ip)
+                    l.warning("Filling memory at %#x with %d unconstrained bytes referenced from %s (%s)", addr, num_bytes, refplace_int_s, refplace_str)
+                else:
+                    if addr == self.state.arch.ip_offset:
+                        refplace_int_s = "0"
+                        refplace_str = "symbolic"
+                    else:
+                        refplace_str = "unknown"
+                        if not self.state._ip.symbolic:
+                            refplace_int = self.state.solver.eval(self.state._ip)
+                            refplace_int_s = "%#x" % refplace_int
+                            if self.state.project:
+                                refplace_str = self.state.project.loader.describe_addr(refplace_int)
                         else:
-                            refplace_str = "unknown"
+                            refplace_int_s = repr(self.state._ip)
                     reg_str = self.state.arch.translate_register_name(addr, size=num_bytes)
-                    l.warning("Filling register %s with %d unconstrained bytes referenced from %#x (%s)", reg_str, num_bytes, refplace_int, refplace_str)
+                    l.warning("Filling register %s with %d unconstrained bytes referenced from %s (%s)", reg_str, num_bytes, refplace_int_s, refplace_str)
 
         # this is an optimization to ensure most operations in the future will deal with leaf ASTs (instead of reversed
         # ASTs)
