@@ -367,7 +367,7 @@ class SimMemory(SimStatePlugin):
     def _resolve_location_name(self, name, is_write=False):
 
         # Delayed load so SimMemory does not rely on SimEngines
-        from ..engines.vex.ccall import _get_flags
+        from angr.engines.vex.claripy.ccall import _get_flags
 
         if self.category == 'reg':
             if self.state.arch.name in ('X86', 'AMD64'):
@@ -378,13 +378,13 @@ class SimMemory(SimStatePlugin):
                 elif name in ('flags', 'eflags', 'rflags'):
                     # we tweak the state to convert the vex condition registers into the flags register
                     if not is_write: # this work doesn't need to be done if we're just gonna overwrite it
-                        self.store('cc_dep1', _get_flags(self.state)[0]) # TODO: can constraints be added by this?
+                        self.store('cc_dep1', _get_flags(self.state))
                     self.store('cc_op', 0) # OP_COPY
                     return self.state.arch.registers['cc_dep1'][0], self.state.arch.bytes
             if is_arm_arch(self.state.arch):
                 if name == 'flags':
                     if not is_write:
-                        self.store('cc_dep1', _get_flags(self.state)[0])
+                        self.store('cc_dep1', _get_flags(self.state))
                     self.store('cc_op', 0)
                     return self.state.arch.registers['cc_dep1'][0], self.state.arch.bytes
 
@@ -514,6 +514,9 @@ class SimMemory(SimStatePlugin):
         elif size_e is None:
             size_e = self.state.solver.BVV(data_e.size() // self.state.arch.byte_width, self.state.arch.bits)
 
+        if endness is None:
+            endness = self.endness
+
         if len(data_e) % self.state.arch.byte_width != 0:
             raise SimMemoryError("Attempting to store non-byte data to memory")
         if not size_e.symbolic and (len(data_e) < size_e*self.state.arch.byte_width).is_true():
@@ -528,11 +531,13 @@ class SimMemory(SimStatePlugin):
                     reg_write_length=size_e,
                     reg_write_expr=data_e,
                     reg_write_condition=condition_e,
+                    reg_write_endness=endness,
                 )
                 addr_e = self.state._inspect_getattr('reg_write_offset', addr_e)
                 size_e = self.state._inspect_getattr('reg_write_length', size_e)
                 data_e = self.state._inspect_getattr('reg_write_expr', data_e)
                 condition_e = self.state._inspect_getattr('reg_write_condition', condition_e)
+                endness = self.state._inspect_getattr('reg_write_endness', endness)
             elif self.category == 'mem':
                 self.state._inspect(
                     'mem_write',
@@ -541,11 +546,13 @@ class SimMemory(SimStatePlugin):
                     mem_write_length=size_e,
                     mem_write_expr=data_e,
                     mem_write_condition=condition_e,
+                    mem_write_endness=endness,
                 )
                 addr_e = self.state._inspect_getattr('mem_write_address', addr_e)
                 size_e = self.state._inspect_getattr('mem_write_length', size_e)
                 data_e = self.state._inspect_getattr('mem_write_expr', data_e)
                 condition_e = self.state._inspect_getattr('mem_write_condition', condition_e)
+                endness = self.state._inspect_getattr('mem_write_endness', endness)
 
         # if the condition is false, bail
         if condition_e is not None and self.state.solver.is_false(condition_e):
@@ -748,22 +755,26 @@ class SimMemory(SimStatePlugin):
             size = self.state.arch.bits // self.state.arch.byte_width
             size_e = size
 
+        endness = self.endness if endness is None else endness
+
         if _inspect:
             if self.category == 'reg':
                 self.state._inspect('reg_read', BP_BEFORE, reg_read_offset=addr_e, reg_read_length=size_e,
-                                    reg_read_condition=condition_e
+                                    reg_read_condition=condition_e, reg_read_endness=endness,
                                     )
                 addr_e = self.state._inspect_getattr("reg_read_offset", addr_e)
                 size_e = self.state._inspect_getattr("reg_read_length", size_e)
                 condition_e = self.state._inspect_getattr("reg_read_condition", condition_e)
+                endness = self.state._inspect_getattr("reg_read_endness", endness)
 
             elif self.category == 'mem':
                 self.state._inspect('mem_read', BP_BEFORE, mem_read_address=addr_e, mem_read_length=size_e,
-                                    mem_read_condition=condition_e
+                                    mem_read_condition=condition_e, mem_read_endness=endness,
                                     )
                 addr_e = self.state._inspect_getattr("mem_read_address", addr_e)
                 size_e = self.state._inspect_getattr("mem_read_length", size_e)
                 condition_e = self.state._inspect_getattr("mem_read_condition", condition_e)
+                endness = self.state._inspect_getattr('mem_read_endness', endness)
 
         if (
             o.UNDER_CONSTRAINED_SYMEXEC in self.state.options and
@@ -802,7 +813,6 @@ class SimMemory(SimStatePlugin):
             self.state.uninitialized_access_handler(self.category, normalized_addresses, size, r, self.state.scratch.bbl_addr, self.state.scratch.stmt_idx)
 
         # the endianess
-        endness = self.endness if endness is None else endness
         if endness == "Iend_LE":
             r = r.reversed
 
